package PureMessage::RPC::Request;
use strict;
use warnings;
use Exporter;
use PureMessage::RPC::Config qw( %config);
use PureMessage::RPC::Response;
use PureMessage::Logger qw( LOG_DEBUG LOG_INFO LOG_NOTICE LOG_WARNING LOG_ERR LOG_CRIT LOG_ALERT LOG_EMERG); 
our @ISA = qw(Exporter);

# pass a no_auto_commit => 1 to cause all call methods to just return the
# response object without having the result printed to STDOUT.
sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my %self = @_;
    return bless( \%self, $class);
}

sub dispatch_error {
   my $self = shift;
   my $r = PureMessage::RPC::Response->new( _request => $self, buffer => 1);
   $r->raise_error(@_);
   $r->commit() unless $self->{no_auto_commit};
   return $r;
}

sub call {
    my $self = shift;
    my $class = shift || 'PureMessage::RPC::Response';
    my $method = shift || 'doc';
    if (  $class =~ /[^a-zA-Z0-9:\-_]/ ){
        #QuickFix for PMX-67
        my $r = PureMessage::RPC::Response->new( _request => $self, buffer => 1);
        $self->log_error("Possible attack pattern found (REMOTE_ADDR:". $ENV{'REMOTE_ADDR'}."):". $class);
        $r->raise_error("Service not found: UNDEF");
        $r->commit() unless $self->{no_auto_commit};
        return $r;
    }
    $class = "PureMessage::RPC::Response::$class" if $class !~ /::/;
    my @data = @_;
    my $r;
    {
      no warnings;
      eval "require $class";
    };
    if ( $@ ) {
        my $err = $@;
        $r = PureMessage::RPC::Response->new( _request => $self, buffer => 1);
        $self->log("Service not found: $err");
        $r->raise_error("Service not found: $err");
    }
    else {
        { # for local
          ## TODO this tie might be very expensive and these STDOUT redirects
          #have dupious usability when not debugging. Either make this less
          #sucky or remove them.

            use IO::Scalar;
            no warnings qw(once); # turn off only used once warning for OLDOUT
           
            my $output = undef;
           
            if (!$self->{nobuffer}) {
                require IO::Scalar;
                open OLDOUT,">&STDOUT";
                local *STDOUT;
                tie *STDOUT, "IO::Scalar", \$output;
            }
            
            $r = $class->new( _request => $self, buffer => 1);
            $r->{REQUEST} = $self;
            
            unless($r->isa('PureMessage::RPC::Response')) {
             die "Invalid Package '$class'";
            }

            if ( $r->can($method) ) {
                eval { $r->$method(@data) };
                my $err = $@;

                if (!$self->{nobuffer}) {
                  untie *STDOUT;
                  open STDOUT,">&OLDOUT";
                }
                if ( $err ) {
                   # XXX This isn't working in all cases for some reason ... more
                   # investigation is needed.
                   $r->raise_error("RPC call failed: $err") unless $r->dispatched_errors();
                   $r->log("RPC call failed: $err");
                }
                else {
                   if ( ! $r->{done_headers} ) { 
                       # no data sent by the method
                       $r->dispatch({'UNDEF'=>1})
                   }
                   $r->serve("\cD"); # EOF
                   $r->commit() if !$self->{no_auto_commit};
                   die "Spurious output $output" if $output; #TODO figure out what is reasonable to do given this error
                }
            }
            else {
                
                if (!$self->{nobuffer}) {
                  untie *STDOUT;
                  open STDOUT,">&OLDOUT";
                }
                $r->raise_error("Method $method is not implemented.");
            }

        }
    }
   $r->commit() unless $self->{no_auto_commit};
   return $r;
}



sub _init_log {
  my $self = shift;
  my $log = shift;
  $log = $log || 'audit_log';
  my $location = $config{$log};
  $self->{$log} = PureMessage::Logger->new( ident => 'rpc', log_to=>$location,debug_level => $config{debug_level});
}

sub _log {
  my ($self,$log,$PRIORITY,@msg) = @_;
  $self->_init_log($log) unless $self->{$log};
  $self->{$log}->log($PRIORITY,join(' ',@msg));
  
}

sub _logger {
  my $self = shift;
  return $self->{audit_log} ? $self->{audit_log} : $self->_init_log();
}

sub log_debug {
  my $self = shift;
  $self->_log('log_debug',LOG_ERR,@_);
}

sub log_error {
  my $self = shift;
  $self->_log('error_log',LOG_ERR,@_);
}

sub log {
  my $self = shift;
  $self->_log('audit_log',LOG_INFO,@_);
}



1;
